\\ Copyright 2016, 2017, 2018, 2019, 2020, 2021 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  If not, see <http://www.gnu.org/licenses/>.


\\ Exercise recurrence-guess.gp by running it on some vectors of numbers,
\\ printing the various results.

allocatemem(200*10^6);
default(strictargs,1);
default(recover,0);
read("recurrence-guess.gp");

print("GP version ",version())

test_verbose = 0;
test_count = 0;
test_equal_noprint(got,want,name="") =
{
  test_count++;
  \\ print("got  ",got);
  \\ print("want ",got);
  if(got!=want,
     if(name!="",print1(name,": "));
     print("oops\ngot =",got,"\nwant=",want);
     quit(1));
}
test_equal(got,want,name="") =
{
  test_equal_noprint(got,want,name);
  if(test_verbose,
     if(name=="",print("ok"),print(name,"  ok")));
}

x=123456;  \\ be safe against x as a global too

\\-----------------------------------------------------------------------------
\\ recurrence_guess()

sqrt5 = quadgen(4*5);
test_equal(sqrt5^2, 5);
phi = quadgen(5);    \\ (1 + sqrt(5))/2  D=5
test_equal((2*phi-1)^2, 5);

vm(v,i) = v[(i% #v)+1];
try(v,name="") =
{
  print("-----------------------------------------");
  if(name!="", print("try: ",name));
  recurrence_guess(v);
}

\\ num or den with a power, but otherwise one factor
recurrence_guess_INTERNAL_print_gf((1-'x)^2/(1-6*'x+3*'x^2-'x^3));
recurrence_guess_INTERNAL_print_gf((1-'x)/(1-6*'x+3*'x^2-'x^3)^2);
\\ and without power
recurrence_guess_INTERNAL_print_gf(-(1-'x)/(1-6*'x+3*'x^2-'x^3));

try( vector(20,n,n--; 0), "zeros");
try( vector(20,n,n--; 1), "ones");

try( vector(50,k,k--; -'y*'z), "-y*z");
try( vector(50,k,k--; 'y*k^2*vm([1,0,0,0],k)) );
try( vector(50,k,k--; 1 + 2^k*'y + 3^k*'y^2), "2^k,3^k times 'y");
try( vector(50,n,n--; (1+'y)^n), "binomial coeffs");
try( vector(50,n,n--; y^n/(1+y-y^2)), "Fibonaccis shifted");

recurrence_guess_INTERNAL_print_gf((33 - 151*'x + 264*'x^2 - 210*'x^3 + 65*'x^4)/(1 - 'x)^5);
recurrence_guess_INTERNAL_print_gf((3 - 6*'x + 4*'x^2)/(1 - 'x)^5 );
try(vector(100,n,n+=4; (n-3)*(n-4)*(n^2-11*n+66)/24), "Tplus dodgy printout");

print("n*vm[1,0]");
try( vector(10,n,n--; n*vm([1,0],n)) );
try( vector(10,n,n--; n*vm([0,1],n)) );
try( recurrence_guess_INTERNAL_gf_terms( 1/(1 + 'x^2)^2, 100));
try( recurrence_guess_INTERNAL_gf_terms( 1/(1 - 'x^2)^2, 100));
try( concat([0,0,0,0,0],vector(10,k,k--; 2^k)) );  \\ initial exceptions

try( vector(32,k,k--; vm([1,2,3,4,5,6,7,8],k)), "periodic 1 to 8");
try( I*vector(32,k,k--; vm([1,2,3,4,5,6,7,8],k)), "periodic 1 to 8 with I" );

try( vector(10,n,n--; phi^n), "phi^n");
try( vector(10,n,n--; fibonacci(n)), "fibonacci");
try( vector(10,n,n--; fibonacci(n))*phi, "phi*fibonacci");
try( vector(50,n,n--; if(n<2,99,fibonacci(n))), \
    "fibonacci with initial exceptions, per docs");
try( vector(10,n,n--; 3^n + 4*2^n*fibonacci(n)) );

try( vector(20,n,n--; n!), "factorial, not linear recurrence");

try( vector(10,n,n--; 3^n + n*2^n) );
try( vector(10,n,n--; 2^n+3^n) );
try( vector(10,n,n--; vm([7,5,3,11],n)) );
try( vector(10,n,n--; vm([7,5,3,11],n)*2^n) );
try( vector(10,n,n--; vm([7,5,3],n)*3^n) );
try( vector(50,n,n--; vm([7,5,3,11],n)*2^n + 3^n*I) );
try( vector(50,n,n--; n^2 * 2^n) );
try( vector(50,n,n--; n^5 + 1) );

print("-2 and -3 powers");
try( vector(10,n,n--; (-2)^n+(-3)^n) );

try( recurrence_guess_INTERNAL_gf_terms(1/(1-'x^2),20) );
try( recurrence_guess_INTERNAL_gf_terms(1/(1+'x^2),20) );
try( recurrence_guess_INTERNAL_gf_terms(1/(1-2*'x),20) );
try( recurrence_guess_INTERNAL_gf_terms(1/(1-2*'x^2),20) );
try( vector(50,n,n--; 10*n^3 + 11*n^2 + 12*n + 13) );
try( vector(50,n,n--; 10*n^3 * 7^n) );
try( vector(50,n,n--; n^2) );
try( [1, 3, 7, 13, 21, 31, 43, 57] );
try( vector(50,n,n--; (n^2+1)*3^n) );

try( vector(50,n,n--; (3+4*I)^n) );
try( vector(50,n,n--; real((3+4*I)^n)) );
try( vector(50,n,n--; imag((3+4*I)^n)) );
try( [1,2,3, 10,20,30, 100,200,300] );

try( [0, 0, 1, 10, 50, 214, 802, 2802, 9275, 29580, 91668, 277924, 828092, 2433140, 7067885, 20337318, 58054534, 164602410, 463990190, 1301338150, 3633753815] );
try( quadgen(20)*[0, 0, 1, 10, 50, 214, 802, 2802, 9275, 29580, 91668, 277924, 828092, 2433140, 7067885, 20337318, 58054534, 164602410, 463990190, 1301338150, 3633753815], "with sqrt 5");

try( vector(50,n,n--; (1+I)^n) );
try( vector(50,n,n--; (1+I)^n + (1-I)^n) );
print("I * ((1+I)^n + (1-I)^n)");
try( vector(50,n,n--; I * ((1+I)^n + (1-I)^n)) );

\\ exercising some expon factor *n in periodic part
try( [ 0, 0, 0, 2, 4, 30, 64, 106, 284, 622, 1528, 3506, 7124, 15702, 34160, 73786, 160076, 337374, 714856, 1518018, 3201988, 6754534, 14167136 ], \
    "from ComplexMinus negate bits flipped within level k");
try( recurrence_guess_INTERNAL_gf_terms \
    ((82*'x + 28)/(900*'x^4 + 1800*'x^3 + 1800*'x^2 + 900*'x + 225), 50), \
    "82*x + 28 etc");
try( recurrence_guess_INTERNAL_gf_terms(1/(1-4*'x^4)^2, 20), "1/(1-4*x^4)^2");

try( vector(10,n,n--; 's^n), "symbol s^n");

try( vector(10,n,n--; (y+2)^(n-2) * (1 + y*(y+4) - n*(1+2*y))), \
    "poly coeffs, star-n distance polynomials");


\\-----------------------------------------------------------------------------
\\ no globals left over

print("-----------------------------------------");

kill(x);
\\ width=1    \\ uncomment to provoke an error

{
  my(test_symbols=readstr("test-symbols.txt"));
  \\ print(test_symbols);
  for(test_i=1,#test_symbols,
     my(test_symbol=test_symbols[test_i]);
     test_equal_noprint(eval(test_symbol),
                        eval(Str("'"test_symbol)),
                        Str("global variable "test_i
                            " should not be set: \""test_symbol"\"")));
}

print(test_count," tests ok");
