\\ Copyright 2016, 2017, 2018, 2020, 2021, 2024 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  If not, see <http://www.gnu.org/licenses/>.

default(strictargs,1);
default(recover,0);
read("recurrence-guess.gp");

print("GP version ",version())

test_verbose = 0;
test_count = 0;
test_equal_noprint(got,want,name="") =
{
  test_count++;
  \\ print("got  ",got);
  \\ print("want ",got);
  if(got!=want,
     print("oops ",name);
     print("got  ",got);
     print("want ",want);
     quit(1));
}
test_equal(got,want,name="") =
{
  test_equal_noprint(got,want,name);
  if(test_verbose,
     if(name=="",print("ok"),print(name,"  ok")));
}

x=123456;  \\ be safe against x as a global too
y=654321;
z=987654;

\\-----------------------------------------------------------------------------
\\ recurrence_guess_INTERNAL_order_and_sig()

{
  \\ f coeffs from low to high
  test_equal(recurrence_guess_INTERNAL_order_and_sig([10,20,30]), [3, "30,20,10"]);
  test_equal(recurrence_guess_INTERNAL_order_and_sig([0,0,10,20,30]), [3, "30,20,10"]);
  test_equal(recurrence_guess_INTERNAL_order_and_sig([0,10,20,30,0]), [4, "0,30,20,10"]);
  test_equal(recurrence_guess_INTERNAL_order_and_sig([0,1,0]), [2, "0,1"]);

  \\ all 0s
  test_equal(recurrence_guess_INTERNAL_order_and_sig([]),    [1,"0"]);
  test_equal(recurrence_guess_INTERNAL_order_and_sig([0]),   [1,"0"]);
  test_equal(recurrence_guess_INTERNAL_order_and_sig([0,0]), [1,"0"]);
}


\\-----------------------------------------------------------------------------
\\ recurrence_guess_INTERNAL_formula()

{
  \\ f coeffs from low to high
  test_equal(recurrence_guess_INTERNAL_formula([1,1]), "a(n) = a(n-1) + a(n-2)");
  test_equal(recurrence_guess_INTERNAL_formula([1,-1]), "a(n) = -a(n-1) + a(n-2)");
  test_equal(recurrence_guess_INTERNAL_formula([1,-9]), "a(n) = -9*a(n-1) + a(n-2)");
  test_equal(recurrence_guess_INTERNAL_formula([1,-9*I]), "a(n) = -9*I*a(n-1) + a(n-2)");
  test_equal(recurrence_guess_INTERNAL_formula([1+I, 1]), "a(n) = a(n-1) + (1 + I)*a(n-2)");

  test_equal(recurrence_guess_INTERNAL_formula([1]),
             "a(n) = a(n-1)");
  test_equal(recurrence_guess_INTERNAL_formula([0,1]),
             "a(n) = a(n-1)",
             "recurrence_guess_INTERNAL_formula() [0,1]");
  test_equal(recurrence_guess_INTERNAL_formula([0,0,1]),
             "a(n) = a(n-1)");
  test_equal(recurrence_guess_INTERNAL_formula([0,0, -10,-20,-30]),
             "a(n) = -30*a(n-1) - 20*a(n-2) - 10*a(n-3)");
}


\\-----------------------------------------------------------------------------
\\ Comments Example

{
  my(g=1/(1 - 'x - 2*'x^3),
     v=Vec(g + O('x^15)));
  \\ recurrence_guess(v);
  
  test_equal(recurrence_guess_values_to_gf(v),
             g,
             "example recurrence_guess_values_to_gf()");

  test_equal(v[8], 23);
  test_equal([2, 0, 1] * v[5..7]~,
             v[8],
             "example scalar product");

  my(A077949_samps=[1, 1, 1, 3, 5, 7, 13, 23, 37, 63, 109, 183, 309, 527, 893]);
  test_equal(v, A077949_samps,
             "example A077949");

  my(f=[2, 0, 1]);
  test_equal(recurrence_guess_INTERNAL_formula(f),
             "a(n) = a(n-1) + 2*a(n-3)");
  test_equal(recurrence_guess_INTERNAL_order_and_sig(f),
             [3, "1,0,2"]);
}


\\-----------------------------------------------------------------------------

{
  test_equal(1/87 * (3 - 6*'x + 4*'x^2) / (1 - 'x)^5,
             1/87 * (3 - 6*'x + 4*'x^2)/(1 - 'x)^5);
  test_equal(  4/87/(1 - 'x)^3
             - 2/87/(1 - 'x)^4
             + 1/87/(1 - 'x)^5
             ,
             1/87 * (3 - 6*'x + 4*'x^2)/(1 - 'x)^5);
}


\\-----------------------------------------------------------------------------
\\ recurrence_guess_INTERNAL_pol_lowest_coeff()

test_equal(recurrence_guess_INTERNAL_pol_lowest_coeff(9*'x+20), 20, \
           "recurrence_guess_INTERNAL_pol_lowest_coeff()");
test_equal(recurrence_guess_INTERNAL_pol_lowest_coeff(9*'x^6+15*'x^2), 15);

test_equal(recurrence_guess_INTERNAL_pol_lowest_coeff(5*'y^2+6*'x, 'x), 5*'y^2);
test_equal(recurrence_guess_INTERNAL_pol_lowest_coeff(5*'y^2+6*'x, 'y), 6*'x);
test_equal(recurrence_guess_INTERNAL_pol_lowest_coeff(5*'y^2+6*'x*'y, 'y), 6*'x);

\\-----------------------------------------------------------------------------
\\ recurrence_guess_INTERNAL_parens_for_power()

test_equal(recurrence_guess_INTERNAL_want_parens_for_power(1),   0);
test_equal(recurrence_guess_INTERNAL_want_parens_for_power(1/2), 1);
test_equal(recurrence_guess_INTERNAL_want_parens_for_power(9/2), 1);
test_equal(recurrence_guess_INTERNAL_want_parens_for_power(I),   0);
test_equal(recurrence_guess_INTERNAL_want_parens_for_power(-1),  1);
test_equal(recurrence_guess_INTERNAL_want_parens_for_power(-I),  1);
test_equal(recurrence_guess_INTERNAL_want_parens_for_power(1+I), 1);

test_equal(recurrence_guess_INTERNAL_parens_for_power(1), 1);
test_equal(recurrence_guess_INTERNAL_parens_for_power(-1), "(-1)");
test_equal(recurrence_guess_INTERNAL_parens_for_power(I), I);
test_equal(recurrence_guess_INTERNAL_parens_for_power(-I), "(-I)");
test_equal(recurrence_guess_INTERNAL_parens_for_power(2), 2);
test_equal(recurrence_guess_INTERNAL_parens_for_power(-2), "(-2)");
test_equal(recurrence_guess_INTERNAL_parens_for_power(2*I), "(2*I)");
test_equal(recurrence_guess_INTERNAL_parens_for_power(-2*I), "(-2*I)");
test_equal(recurrence_guess_INTERNAL_parens_for_power(-2+I), "(-2 + I)");
test_equal(recurrence_guess_INTERNAL_parens_for_power(2+quadgen(20)), \
           Str("(",2+quadgen(20),")"));
test_equal(recurrence_guess_INTERNAL_parens_for_power('x), 'x, "gf x");
test_equal(recurrence_guess_INTERNAL_parens_for_power(2*'x), "(2*x)");
test_equal(recurrence_guess_INTERNAL_parens_for_power(-'x), "(-x)");
test_equal(recurrence_guess_INTERNAL_parens_for_power((2+I)*'x), \
           Str("(",(2+I)*'x,")"));
test_equal(recurrence_guess_INTERNAL_parens_for_power('x^2), "(x^2)");
test_equal(recurrence_guess_INTERNAL_parens_for_power(Pol(1)), 1);
test_equal(recurrence_guess_INTERNAL_parens_for_power(Pol(-1)), "(-1)");


\\-----------------------------------------------------------------------------
\\ recurrence_guess_INTERNAL_want_parens_for_multiply()

test_equal(recurrence_guess_INTERNAL_want_parens_for_multiply(0), 0, "0");
test_equal(recurrence_guess_INTERNAL_want_parens_for_multiply(1), 0, "1");
test_equal(recurrence_guess_INTERNAL_want_parens_for_multiply(123), 0, "123");
test_equal(recurrence_guess_INTERNAL_want_parens_for_multiply(1+'x), 1, "1+x");
test_equal(recurrence_guess_INTERNAL_want_parens_for_multiply((1+'y)*'x), 1);

\\ recurrence_guess__parens_for_multiply()
test_equal(recurrence_guess_INTERNAL_parens_for_multiply(1), 1, "1");
test_equal(recurrence_guess_INTERNAL_parens_for_multiply(-1), -1);
test_equal(recurrence_guess_INTERNAL_parens_for_multiply(I), I);
test_equal(recurrence_guess_INTERNAL_parens_for_multiply(-I), -I);
test_equal(recurrence_guess_INTERNAL_parens_for_multiply(2), 2);
test_equal(recurrence_guess_INTERNAL_parens_for_multiply(-2), -2);
test_equal(recurrence_guess_INTERNAL_parens_for_multiply(2*I), 2*I);
test_equal(recurrence_guess_INTERNAL_parens_for_multiply(-2*I), -2*I);
test_equal(recurrence_guess_INTERNAL_parens_for_multiply(1+I), "(1 + I)");
test_equal(recurrence_guess_INTERNAL_parens_for_multiply(-1+I), "(-1 + I)");
test_equal(recurrence_guess_INTERNAL_parens_for_multiply(1+quadgen(20)), \
           "(1 + w)");


\\-----------------------------------------------------------------------------
\\ recurrence_guess_INTERNAL_is_negative()

test_equal(recurrence_guess_INTERNAL_is_negative(0), 0);
test_equal(recurrence_guess_INTERNAL_is_negative(1), 0);
test_equal(recurrence_guess_INTERNAL_is_negative(-1), 1);
test_equal(recurrence_guess_INTERNAL_is_negative(I), 0);
test_equal(recurrence_guess_INTERNAL_is_negative(-I), 1);
test_equal(recurrence_guess_INTERNAL_is_negative(-'y), 1);
test_equal(recurrence_guess_INTERNAL_is_negative(-'y*'z), 1);

\\-----------------------------------------------------------------------------

test_equal((5248/225*x^13 - 8704/225*x^12 + 768/25*x^11 - 1664/225*x^10 - 3104/225*x^9 + 64/3*x^8 - 384/25*x^7 + 832/225*x^6 + 568/225*x^5 - 256/75*x^4 + 48/25*x^3 - 104/225*x^2 - 2/15*x + 28/225)/(-16*x^8 + 1)^2, \
           (28/225 + 82/225*x)/(1 + 2*x + 2*x^2)^2);

test_equal(concat(Mat([123;1]), ['x,1;'y,2;'z,3]~)~, \
           [123,1; 'x,1; 'y,2; 'z,3]);

{
  my(w=quadgen(4*5));  \\ sqrt5
  test_equal((w*x^2 + 4*w*x^3 - 3*w*x^4 + w*x^6)/(1 - 6*x + 7*x^2 + 16*x^3 - 27*x^4 - 18*x^5 + 29*x^6 + 12*x^7 - 9*x^8 - 2*x^9 + x^10)
             ,
             1/25*w * (5*x)^2 * (1 + 4*x - 3*x^2 + x^4) / ( (1 + x)^2 * (1 - (3/2 - 1/2*w)*x)^2 * (1 - (1/2 + 1/2*w)*x)^2 * (1 - (3/2 + 1/2*w)*x)^2 * (1 - (1/2 - 1/2*w)*x)^2 ));
}

\\-----------------------------------------------------------------------------
\\ initial exceptions and values length

\\ Fibonacci order 2 is 2*2+1=5 values required
test_equal(recurrence_guess_values_to_factors(vector(4,n,fibonacci(n))), []);
test_equal(recurrence_guess_values_to_factors(vector(5,n,fibonacci(n))), [1,1]);

\\ Fibonacci order 2 is 2*2+1=5 values required
test_equal(recurrence_guess_values_to_factors(concat(99,vector(5,n,fibonacci(n)))), []);
test_equal(recurrence_guess_values_to_factors(concat(99,vector(6,n,fibonacci(n)))), [0,1,1]);


\\-----------------------------------------------------------------------------
\\ recurrence_guess_INTERNAL_pol_periodic()

test_equal((1-'x^8) % (1 + I*'x^2), 0);
test_equal((1+'x^4) % (1 + I*'x^2), 0);
test_equal(recurrence_guess_INTERNAL_pol_periodic(1 + I*'x^2), 1-'x^8, \
           "pol_periodic 1 + I*x^2");

test_equal(recurrence_guess_INTERNAL_pol_periodic(1-2*'x), 1-2*'x, \
           "pol_periodic 1-2*x");
test_equal(recurrence_guess_INTERNAL_pol_periodic(-1+2*'x), 1-2*'x);
test_equal(recurrence_guess_INTERNAL_pol_periodic(1-2*'x^2), 1-2*'x^2);
test_equal(recurrence_guess_INTERNAL_pol_periodic(1+2*'x), 1-4*'x^2, \
           "pol_periodic 1+2*x");
test_equal((1-4*'x^2) % (1+2*'x), 0);
test_equal((1+2*'x)*(1-2*'x), 1-4*'x^2);

{
  for(e=2,20,
     for(c=1,8,
        my(n=1-c*'x^e,
           f=factor(n),
           len=matsize(f)[1]);
        if(len<2,next());
        for(i=1,len,
           my(p=f[i,1],
              got=recurrence_guess_INTERNAL_pol_periodic(p),
              good = got!=0 && n%got==0);
           if(!good, print("c="c" e="e"  n="n"\nf="f"\np="p"\ngot="got));
           test_equal_noprint(good,1, "pol_periodic"))));
}

\\-----------------------------------------------------------------------------
\\ recurrence_guess_INTERNAL_besp_to_gf()

test_equal(recurrence_guess_INTERNAL_besp_to_gf(2,0,1,[1/32]), \
           1/32 * 1/(1-2*'x),  "1/32 * 2^n");

test_equal(recurrence_guess_INTERNAL_besp_to_gf(2,0,1,[1]), \
           1/(1-2*'x),  "2^n");

test_equal(recurrence_guess_INTERNAL_besp_to_gf(1,0,1,[0,1]), \
           (0+1*'x)/(1-'x^2),  "0,1 repeating");

test_equal(recurrence_guess_INTERNAL_besp_to_gf(2,1,1,[1]), \
           2*'x/(1-2*'x)^2,  "n*2^n");

test_equal(recurrence_guess_INTERNAL_besp_to_gf(1,0,1,['y]), \
           'y/(1-'x),  "y");

test_equal(recurrence_guess_INTERNAL_besp_to_gf(1,2,1,['y]), \
           ('y*'x + 'y*'x^2)/(1-'x)^3,  "y*n^2");

test_equal(recurrence_guess_INTERNAL_besp_to_gf(1,2,1,['y,0,0,0]), \
           16*('y*'x^8 + 'y*'x^4)/(1-'x^4)^3, \
           "n^2*vector_modulo([y,0,0,0])");



\\-----------------------------------------------------------------------------
\\ recurrence_guess_INTERNAL_periodic_vector_reduce()

test_equal(Vecrev(divisors(12))[^1], [6,4,3,2,1]);
test_equal(recurrence_guess_INTERNAL_periodic_vector_reduce([0]), [0])
test_equal(recurrence_guess_INTERNAL_periodic_vector_reduce([3,3]), [3])
test_equal(recurrence_guess_INTERNAL_periodic_vector_reduce([1,1]), [1])
test_equal(recurrence_guess_INTERNAL_periodic_vector_reduce([7,3,7,3]), [7,3])
test_equal(recurrence_guess_INTERNAL_periodic_vector_reduce([7,3,7]), [7,3,7])
test_equal(recurrence_guess_INTERNAL_periodic_vector_reduce([7,3,7,3,7,3]), [7,3])
test_equal(recurrence_guess_INTERNAL_periodic_vector_reduce([1,2,3,1,2,3]), [1,2,3])

\\-----------------------------------------------------------------------------
test_equal(recurrence_guess_values_to_gf([1,2,4,8,16]), \
           1/(1-2*'x))

\\-----------------------------------------------------------------------------

test_equal([1,1,1,1,1], vector(5,n,n--; 1^floor(n/2)));


\\-----------------------------------------------------------------------------
{
  my(w=quadgen(20));
  test_equal(
  (w*'x^2 + 4*w*'x^3 - 3*w*'x^4 + w*'x^6)/(1 - 6*'x + 7*'x^2 + 16*'x^3 - 27*'x^4 - 18*'x^5 + 29*'x^6 + 12*'x^7 - 9*'x^8 - 2*'x^9 + 'x^10)
  ,
  16/25*w/(1 + 'x)
  - 1/5*w/(1 + 'x)^2
  + (111/20 - 247/100*w)/(1 - (3/2 - 1/2*w)*'x)
  + ((-11/10 + 1/2*w))/(1 - (3/2 - 1/2*w)*'x)^2
  + (77/20 + 27/20*w)/(1 - (1/2 + 1/2*w)*'x)
  + ((1 + 2/5*w))/(1 - (1/2 + 1/2*w)*'x)^2
  + (-111/20 - 247/100*w)/(1 - (3/2 + 1/2*w)*'x)
  + ((11/10 + 1/2*w))/(1 - (3/2 + 1/2*w)*'x)^2
  + (-77/20 + 27/20*w)/(1 - (1/2 - 1/2*w)*'x)
  + ((-1 + 2/5*w))/(1 - (1/2 - 1/2*w)*'x)^2
  );
}

\\-----------------------------------------------------------------------------
\\ 1/(1-x)^k = (n+1)*(n+2)*(n+3)*...*(n+k-1)/(k-1)!    k-1 terms, 1 when k=0

test_equal(recurrence_guess_INTERNAL_gf_terms(1/(1-'x),20), \
           vector(20,n,n--; 1));

test_equal(recurrence_guess_INTERNAL_gf_terms(1/(1-'x)^2,20), \
           vector(20,n,n--; (n+1)));

test_equal(recurrence_guess_INTERNAL_gf_terms(1/(1-'x)^3,20), \
           vector(20,n,n--; (n+1)*(n+2)/2));

test_equal(recurrence_guess_INTERNAL_gf_terms(1/(1-'x)^4,20), \
           vector(20,n,n--; (n+1)*(n+2)*(n+3)/6));

test_equal(recurrence_guess_INTERNAL_gf_terms(1/(1-'x)^5,20), \
           vector(20,n,n--; (n+1)*(n+2)*(n+3)*(n+4)/24));
test_equal(4!,24);

{
  for(k=1,6,
     test_equal(recurrence_guess_INTERNAL_gf_terms(1/(1-'x)^k,20),
                vector(20,n,n--; prod(i=1,k-1, (n+i)) / (k-1)!)));
}


\\-----------------------------------------------------------------------------
\\ recurrence_guess_INTERNAL_pol_num_terms()

test_equal(recurrence_guess_INTERNAL_pol_num_terms(1), 1, "1");
test_equal(recurrence_guess_INTERNAL_pol_num_terms(1+'x), 2, "1+'x");
test_equal(recurrence_guess_INTERNAL_pol_num_terms(1+'x^2), 2);
test_equal(recurrence_guess_INTERNAL_pol_num_terms(1+'x^5), 2);
test_equal(recurrence_guess_INTERNAL_pol_num_terms(1+'x^2+'x^5), 3);

\\-----------------------------------------------------------------------------
\\ recurrence_guess_values_to_factors()

\\ 5 terms for order 2 recurrence
test_equal(recurrence_guess_values_to_factors([5,8,13,21]), [], \
           "4 terms fib");
test_equal(recurrence_guess_values_to_factors([5,8,13,21,34]), [1, 1], \
           "5 terms fib");
test_equal(recurrence_guess_values_to_factors([5,8,13,21,34,55]), [1, 1], \
           "6 terms fib");

test_equal(recurrence_guess_values_to_factors([5,8,13,21, 34+999]), [], \
           "5 terms fib with last wrong");

\\ 7 terms for order 3 recurrence
test_equal(recurrence_guess_values_to_factors([1,3,7,13, 21,31]), []);
test_equal(recurrence_guess_values_to_factors([1,3,7,13, 21,31,43]), [1, -3, 3]);
test_equal(recurrence_guess_values_to_factors([1,3,7,13, 21,31,43,57]), [1, -3, 3]);
test_equal(recurrence_guess_values_to_factors(vector(50,n,n--; real((3+4*I)^n))), [-25, 6]);
test_equal(recurrence_guess_values_to_factors(vector(50,n,n--; imag((3+4*I)^n))), [-25, 6]);
test_equal(recurrence_guess_values_to_factors(vector(20,n,n--; n!)), []);

\\ examples in the docs
test_equal(recurrence_guess_values_to_factors([99, 99, 1, 2, 3, 5, 8, 13, 21]), [0,0,1,1]);
test_equal(recurrence_guess_values_to_factors([1,1, 2,2, 4,4, 8,8]), [2, 0]);
test_equal(recurrence_guess_values_to_factors([1,2,3, 10,20,30, 100,200,300]), [10, 0, 0]);

test_equal(recurrence_guess_values_to_factors(vector(20,n,sum(i=1,n,fibonacci(i)))), [-1, 0, 2]);

\\ order 2 is 4 terms guess plus 7 confirm, so 10 not enough but 11 is enough
recurrence_guess_minimum_confirm=7;
test_equal(recurrence_guess_values_to_factors(vector(10,n,fibonacci(n))), [],    "fib 10, with 7 confirm");
test_equal(recurrence_guess_values_to_factors(vector(11,n,fibonacci(n))), [1,1], "fib 11, with 7 confirm");
recurrence_guess_minimum_confirm=1;


\\-----------------------------------------------------------------------------
\\ recurrence_guess_INTERNAL_ascending_str()

test_equal(recurrence_guess_INTERNAL_ascending_str(0,0), "0");
test_equal(recurrence_guess_INTERNAL_ascending_str(70/(1 - 7*'x)^2, 0), \
           "70/(1 - 7*x)^2");
test_equal(recurrence_guess_INTERNAL_ascending_str(120/(1 - 7*'x)^3, 0), \
           "120/(1 - 7*x)^3");
test_equal(recurrence_guess_INTERNAL_ascending_str((1+I)*'x + I*'x^2 + (1-I)*'x^3, 0), \
           "(1 + I)*x + I*x^2 + (1 - I)*x^3");

test_equal(recurrence_guess_INTERNAL_ascending_str(1,  0), "1",    "one");
test_equal(recurrence_guess_INTERNAL_ascending_str(-1, 0), "-1",   "neg one");
test_equal(recurrence_guess_INTERNAL_ascending_str(1,  1), "+ 1",    "one");
test_equal(recurrence_guess_INTERNAL_ascending_str(-1, 1), "- 1",    "neg one");
test_equal(recurrence_guess_INTERNAL_ascending_str(1+'x,   0), "1 + x",   "1+x");
test_equal(recurrence_guess_INTERNAL_ascending_str(1+'x,  1), "+ 1 + x");
test_equal(recurrence_guess_INTERNAL_ascending_str(1-'x,  0), "1 - x",   "1-x");
test_equal(recurrence_guess_INTERNAL_ascending_str(1-'x,  1), "+ 1 - x", "1-x");
test_equal(recurrence_guess_INTERNAL_ascending_str(-1-'x, 0), "-1 - x",  "-1-x");
test_equal(recurrence_guess_INTERNAL_ascending_str(-1-'x, 1), "- 1 - x", "-1-x");
test_equal(recurrence_guess_INTERNAL_ascending_str(2+3*'x,     0), "2 + 3*x");
test_equal(recurrence_guess_INTERNAL_ascending_str(2+(3+I)*'x, 0), "2 + (3 + I)*x");
test_equal(recurrence_guess_INTERNAL_ascending_str(3+I,       0), "3 + I", \
          "3+I");
test_equal(recurrence_guess_INTERNAL_ascending_str((1+'x)/(1-'x), 0), "(1 + x)/(1 - x)");
test_equal(recurrence_guess_INTERNAL_ascending_str((1+I)/(1-'x), 0), "(1 + I)/(1 - x)");
test_equal(recurrence_guess_INTERNAL_ascending_str('x/(1-'x), 0), "x/(1 - x)");
test_equal(recurrence_guess_INTERNAL_ascending_str(1/(1-'x), 0), "1/(1 - x)");
test_equal(recurrence_guess_INTERNAL_ascending_str(1/'x, 0), "1/x");
test_equal(recurrence_guess_INTERNAL_ascending_str(1/'x^2, 0), "1/x^2");
test_equal(recurrence_guess_INTERNAL_ascending_str(1/((1+I)*'x^2), 0), \
           "(1/2 - 1/2*I)/x^2");

{
  for(a=-2,2,
     for(b=-2,2,
        for(c=-2,2,
           for(d=-2,2,
              for(e=-2,2,
                 for(f=-2,2,
                    my(p = (a+b*I) + (c+d*I)*'x + (e+f*I)*'x^2,
                       s = recurrence_guess_INTERNAL_ascending_str(p, 0),
                       x = 'x, \\ for the eval
                       e = eval(s));
                    test_equal_noprint(e,p, s);

                    if(d||e||f,
                       my(p = (a + b*'x + c*'x^2) / (d + e*'x + f*'x^2),
                          s = recurrence_guess_INTERNAL_ascending_str(p, 0),
                          e = eval(s));
                       test_equal_noprint(e,p, s));
                 ))))));
}

\\-----------------------------------------------------------------------------
\\ recurrence order of n^expon * base^floor(n/spread)
{
  for(base=2,3,
     for(expon=0,5,
        for(spread=1,5,
           my(len=spread*(expon+1),
              periodic=vector(spread,i,i--; 7*i+11),
              v=vector(5*len+20,n,n--;
                       n^expon
                       * base^floor(n/spread)
                       * recurrence_guess_INTERNAL_vector_modulo(periodic,n)),
              f=recurrence_guess_values_to_factors(v));
           if(f==[],error("oops, recurrence not found:\n",
                          "base=",base,
                          " expon=",expon,
                          " spread=",spread,
                          " #v=",#v));
           \\ print(#f" "len);
           test_equal_noprint(#f==len,1))));
}

\\-----------------------------------------------------------------------------
\\ no globals left over

\\ the globals set to dummy values during testing
kill(x);
kill(y);
kill(z);

{
  my(test_symbols=readstr("test-symbols.txt"));
  for(test_i=1,#test_symbols,
     my(test_symbol=test_symbols[test_i]);
     test_equal_noprint(eval(test_symbol),
                        eval(Str("'"test_symbol)),
                        Str("global variable test_symbols[",test_i,
                            "] = \"", test_symbol, "\" should not be set")));
}

\\ Don't let any a-z variables be globals.
{
  my(chars=["a","b","c","d","e","f","g","h","i","j","k","l","m",
            "n","o","p","q","r","s","t","u","v","w","x","y","z"]);
  test_equal_noprint(#chars,26);
  test_equal_noprint(vector(#chars, ind,
                            my(str=chars[ind],
                               ev =eval(str));
                            if(ev==eval(Str("'",str)), 1,
                               print(str," = ",ev); 0)),
                     vector(#chars,i, 1),
                     "no single char variables set");
}
test_equal(values, 'values)
test_equal(zeros, 'zeros)
test_equal(low, 'low)
test_equal(ds, 'ds)
test_equal(glist, 'glist)
test_equal(gparts, 'gparts)
test_equal(debug, 'debug)


\\-----------------------------------------------------------------------------
print(test_count," tests ok");
